//
//  ComposeAvatarImageView.swift
//  Tusker
//
//  Created by Shadowfacts on 8/18/20.
//  Copyright © 2020 Shadowfacts. All rights reserved.
//

import SwiftUI

struct ComposeAvatarImageView: View {
    let url: URL
    @State var request: ImageCache.Request? = nil
    @State var avatarImage: UIImage? = nil
    @ObservedObject var preferences = Preferences.shared
    
    var body: some View {
        image
            .resizable()
            .frame(width: 50, height: 50)
            .cornerRadius(preferences.avatarStyle.cornerRadiusFraction * 50)
            .onAppear(perform: self.loadImage)
            .onDisappear(perform: self.cancelRequest)
    }
    
    private var image: Image {
        if let avatarImage = avatarImage {
            return Image(uiImage: avatarImage)
        } else {
            let imageName: String
            switch preferences.avatarStyle {
            case .circle:
                imageName = "person.crop.circle"
            case .roundRect:
                imageName = "person.crop.square"
            }
            return Image(systemName: imageName)
        }
    }
    
    private func loadImage() {
        request = ImageCache.avatars.get(url) { (data) in
            DispatchQueue.main.async {
                self.request = nil
                if let data = data, let image = UIImage(data: data) {
                    self.avatarImage = image
                }
            }
        }
    }
    
    private func cancelRequest() {
        request?.cancel()
    }
}

struct ComposeAvatarImageView_Previews: PreviewProvider {
    static var previews: some View {
        ComposeAvatarImageView(url: URL(string: "https://social.shadowfacts.net/media/4b481afc591a8f3d11d0f5732e5cb320422dec72d7f223ebb5f35d5d0e821a9c.png")!)
    }
}
